@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: __init__.py 11201 2018-08-12 14:45:23Z Lavender $
+# $Id: __init__.py 11206 2018-08-14 10:39:06Z Lavender $
 #
 # Copyright (c) 2018 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -12,8 +12,8 @@
 # limitations under the License.
 #
 # $Author: Lavender $ (last)
-# $Date: 2018-08-12 23:45:23 +0900 (週日, 12 八月 2018) $
-# $Revision: 11201 $
+# $Date: 2018-08-14 19:39:06 +0900 (週二, 14 八月 2018) $
+# $Revision: 11206 $
 
 import os
 import sys
@@ -384,7 +384,7 @@ def attachShopURLs(settings, urlpatterns):
 
     if cathaybkEnable:
         urlpatterns += patterns('',
-            url(r'^cathaybk/preview/', views.returnCathaybkCreditXml),
+            url(r'^cathaybk-credit/checkxml/', views.returnCathaybkCreditXml),
         )
 
     if ecpayEnable or allpayEnable or spgatewayEnable:
