@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: views.py 11201 2018-08-12 14:45:23Z Lavender $
+# $Id: views.py 11206 2018-08-14 10:39:06Z Lavender $
 #
 # Copyright (c) 2018 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -12,8 +12,8 @@
 # limitations under the License.
 #
 # $Author: Lavender $
-# $Date: 2018-08-12 23:45:23 +0900 (週日, 12 八月 2018) $
-# $Revision: 11201 $ 
+# $Date: 2018-08-14 19:39:06 +0900 (週二, 14 八月 2018) $
+# $Revision: 11206 $ 
 
 import json
 import logging
@@ -110,48 +110,74 @@ views.PaymentDetailsView.get_context_data = get_context_data
 @csrf_exempt
 def returnCathaybkCreditXml(request):
     if request.POST:
-        # get data
-        xml = request.POST.get("strRsXML", None)
-        if not xml:
-            return Http404("Not strRsXM in post data.")
+        try:
+            # get data
+            xml = request.POST.get("strRsXML", None)
+            if not xml:
+                raise ValueError("Not strRsXM in post data.")
 
-        root = etree.fromstring(utf8(xml))
+            root = etree.fromstring(utf8(xml))
 
-        data = {}
+            data = {}
 
-        for child in root:
-            if child.tag == "CAVALUE":
-                data["CAVALUE"] = child.text
-            if child.tag in ["ORDERINFO", "AUTHINFO"]:
-                for child2 in child:
-                    data[child2.tag] = child2.text
+            for child in root:
+                if child.tag == "CAVALUE":
+                    data["CAVALUE"] = child.text
+                if child.tag in ["ORDERINFO", "AUTHINFO"]:
+                    for child2 in child:
+                        data[child2.tag] = child2.text
 
-        # check
-        status = data.get('AUTHSTATUS', None)
-        if not status == '0000':
-            logger.error('%s API ERROR: %s' % (CATHAYBK_CREDIT, request.POST))
-            return redirect('checkout:shipping-address')
+            #record data
+            def parseDate(date):
+                year = date[0:4]
+                month = date[4:6]
+                day = date[6:8]
+                return "%s-%s-%s" % (year, month, day)
 
-        #record data
-        def parseDate(date):
-            year = date[0:4]
-            month = date[4:6]
-            day = date[6:8]
-            return "%s-%s-%s" % (year, month, day)
+            if not data.get("AUTHCODE", ""):
+                tradeNo = "NO_TRADE_NO"
+            else:
+                tradeNo = data.get("AUTHCODE", "NO_TRADE_NO")
 
-        cathaybkCreditTrade = CathaybkCreditTrade.objects.create(
-            tradeNo=data.get("AUTHCODE", ""),
-            tradeAmt=int(data.get("AMOUNT", "0")),
-            merchantId=data.get("STOREID", ""),
-            merchantTradeNo=data.get("ORDERNUMBER", ""),
-            rtnCode=data.get("AUTHSTATUS", ""),
-            paymentDate=parseDate(data.get("AUTHTIME", "11111111")),
-            rtnMsg=data.get("AUTHMSG", ""),
-            cardNo=data.get("CARDNO", ""),
-        )
-        cathaybkCreditTrade.save()
+            cathaybkCreditTrade = CathaybkCreditTrade.objects.create(
+                tradeNo=tradeNo,
+                tradeAmt=int(data.get("AMOUNT", "0")),
+                merchantId=data.get("STOREID", ""),
+                merchantTradeNo=data.get("ORDERNUMBER", ""),
+                rtnCode=data.get("AUTHSTATUS", ""),
+                paymentDate=parseDate(data.get("AUTHTIME", "11111111")),
+                rtnMsg=data.get("AUTHMSG", ""),
+                cardNo=data.get("CARDNO", ""),
+            )
+            cathaybkCreditTrade.save()
 
-        # return xml
+            # check
+            status = data.get('AUTHSTATUS', None)
+            if not status == '0000':
+                raise ValueError("AUTHSTATUS: %s" % status)
+        except Exception as e:
+            logger.error('%s API ERROR: %s\n%s' % (
+                    CATHAYBK_CREDIT, request.POST, str(e)))
+        finally:
+            # return xml
+            scheme = request.scheme
+            domain = request.META['HTTP_HOST']
+
+            url = "%s://%s/%s/preview/" % (scheme, domain, CATHAYBK_CREDIT)
+
+            checkValue = hashlib.md5(
+                domain + settings.CATHAYBK_API_CUBKEY).hexdigest()
+            
+            temp = get_template('shop/payment/CathaybkReturn.xml')
+            context = Context({
+                'checkValue': checkValue,
+                'returnUrl': url,
+            })
+            rtXml = temp.render(context)
+
+            return HttpResponse(rtXml)
+    else:
+        logger.error("returnCathaybkCreditXml run to GET")
         scheme = request.scheme
         domain = request.META['HTTP_HOST']
 
@@ -168,8 +194,6 @@ def returnCathaybkCreditXml(request):
         rtXml = temp.render(context)
 
         return HttpResponse(rtXml)
-    else:
-        return Http404("Not strRsXM in post data.")
 
 class SuccessResponseView2(_PaymentDetailsView):
     preview = True
@@ -214,10 +238,10 @@ class SuccessResponseView2(_PaymentDetailsView):
                 if self.rtnMsg == 'ERROR':
                     msg = _(
                         "There was an error in the payment. Please try again.")
-                    messages.warning(self.request, msg)
+                    messages.error(self.request, msg)
                     logger.error(
                         '%s API ERROR: %s' % (self.apiName, request.POST))
-                    return redirect('checkout:shipping-address')
+                    return redirect('checkout:payment-details')
             
                 # record trade data
                 if self.apiName == ECPAY:
@@ -277,10 +301,10 @@ class SuccessResponseView2(_PaymentDetailsView):
                 if not self.status == 'SUCCESS':
                     msg = _(
                         "There was an error in the payment. Please try again.")
-                    messages.warning(self.request, msg)
+                    messages.error(self.request, msg)
                     logger.error(
                         '%s API ERROR: %s' % (self.apiName, request.POST))
-                    return redirect('checkout:shipping-address')
+                    return redirect('checkout:payment-details')
 
                 # record trade data
                 spgatewayTrade = SpgatewayTrade.objects.create(
@@ -295,11 +319,18 @@ class SuccessResponseView2(_PaymentDetailsView):
                     rtnMsg=self.message,
                 )
                 spgatewayTrade.save()
-            else:
+            elif self.apiName == CATHAYBK_CREDIT:
+                self.status = 'XXXX'
+                msg = _("There was an error in the payment. Please try again.")
+
                 # get data
                 xml = request.POST.get("strRsXML", None)
                 if not xml:
-                    return Http404("Not strRsXM in post data.")
+                    logger.error(
+                        "Not strRsXML in post data.: %s" % request.POST)
+                    messages.error(self.request, msg)
+                    return redirect('checkout:payment-details')
+
                 root = etree.fromstring(utf8(xml))
                 data = {}
                 for child in root:
@@ -309,20 +340,30 @@ class SuccessResponseView2(_PaymentDetailsView):
                         for child2 in child:
                             data[child2.tag] = child2.text
 
-                # get obj
+                # get obj & check
                 self.merchantTradeNo = data.get("ORDERNUMBER", "")
-                cathaybkCreditTrade = get_object_or_404(
-                    CathaybkCreditTrade,
-                    merchantTradeNo=self.merchantTradeNo)       
+                if not CathaybkCreditTrade.objects.filter(
+                        merchantTradeNo=self.merchantTradeNo).exists():
+                    messages.error(self.request, msg)
+                    return redirect('checkout:payment-details')
+                cathaybkCreditTrade = CathaybkCreditTrade.objects.get(
+                    merchantTradeNo=self.merchantTradeNo)
+                self.status = cathaybkCreditTrade.rtnCode
+                if not cathaybkCreditTrade.rtnCode == '0000':
+                    messages.error(
+                        self.request, 
+                        msg + ": %s %s" % (cathaybkCreditTrade.rtnCode, 
+                                           cathaybkCreditTrade.rtnMsg))
+                    return redirect('checkout:payment-details') 
 
                 # data
-                index = self.result.get('MerchantOrderNo').find('BASKETID')
+                index = cathaybkCreditTrade.merchantTradeNo.find('BASKETID')
                 self.basketId = int(
-                    self.result.get('MerchantOrderNo')[index + 8:])
+                    cathaybkCreditTrade.merchantTradeNo[index + 8:])
 
                 self.tradeNo = cathaybkCreditTrade.tradeNo
                 self.tradeAmt = cathaybkCreditTrade.tradeAmt
-            
+
             return self.handle_place_order_submission(request)
         except Exception as e:
             # return msg
@@ -344,6 +385,7 @@ class SuccessResponseView2(_PaymentDetailsView):
                                 self.apiName, request.POST, str(e)))
                         messages.error(self.request, msg1)
                     else:
+                        logger.error(str(e))
                         order = Order.objects.get(basket=request.basket)
                         url = "%s?order_id=%d" % (
                             reverse('checkout:thank-you'), order.id)
@@ -352,7 +394,7 @@ class SuccessResponseView2(_PaymentDetailsView):
                     logger.error(
                         '%s API ERROR:\n%s\n%s' % (
                             self.apiName, request.POST, str(e)))
-                    messages.warning(self.request, msg2)
+                    messages.error(self.request, msg2)
             elif self.apiName == ECPAY:
                 if not self.rtnMsg == 'ERROR':
                     if not ECPayTrade.objects.filter(
@@ -362,6 +404,7 @@ class SuccessResponseView2(_PaymentDetailsView):
                                 self.apiName, request.POST, str(e)))
                         messages.error(self.request, msg1)
                     else:
+                        logger.error(str(e))
                         order = Order.objects.get(basket=request.basket)
                         url = "%s?order_id=%d" % (
                             reverse('checkout:thank-you'), order.id)
@@ -370,8 +413,8 @@ class SuccessResponseView2(_PaymentDetailsView):
                     logger.error(
                         '%s API ERROR:\n%s\n%s' % (
                             self.apiName, request.POST, str(e)))
-                    messages.warning(self.request, msg2)
-            else:
+                    messages.error(self.request, msg2)
+            elif self.apiName == SPGATEWAY:
                 if self.status == 'SUCCESS':
                     if not SpgatewayTrade.objects.filter(
                         merchantTradeNo=self.merchantTradeNo).exists():
@@ -380,6 +423,7 @@ class SuccessResponseView2(_PaymentDetailsView):
                                 self.apiName, request.POST, str(e)))
                         messages.error(self.request, msg1)
                     else:
+                        logger.error(str(e))
                         order = Order.objects.get(basket=request.basket)
                         url = "%s?order_id=%d" % (
                             reverse('checkout:thank-you'), order.id)
@@ -388,9 +432,28 @@ class SuccessResponseView2(_PaymentDetailsView):
                     logger.error(
                         '%s API ERROR:\n%s\n%s' % (
                             self.apiName, request.POST, str(e)))
-                    messages.warning(self.request, msg2)
+                    messages.error(self.request, msg2)
+            elif self.apiName == CATHAYBK_CREDIT:
+                if self.status == '0000':
+                    if not CathaybkCreditTrade.objects.filter(
+                        merchantTradeNo=self.merchantTradeNo).exists():
+                        logger.error(
+                            '%s API ERROR(this need to refund):\n%s\n%s' % (
+                                self.apiName, request.POST, str(e)))
+                        messages.error(self.request, msg1)
+                    else:
+                        logger.error(str(e))
+                        order = Order.objects.get(basket=request.basket)
+                        url = "%s?order_id=%d" % (
+                            reverse('checkout:thank-you'), order.id)
+                        return redirect(url)
+                else:
+                    logger.error(
+                        '%s API ERROR:\n%s\n%s' % (
+                            self.apiName, request.POST, str(e)))
+                    messages.error(self.request, msg2)
 
-            return redirect('checkout:shipping-address')
+            return redirect('checkout:payment-details')
         
         
     def build_submission(self, **kwargs):
@@ -412,7 +475,8 @@ class SuccessResponseView2(_PaymentDetailsView):
         source = Source(source_type=source_type,
                         currency='TWD',
                         amount_allocated=tradeAmt,
-                        amount_debited=tradeAmt) 
+                        amount_debited=tradeAmt,
+                        reference=tradeNo) 
         
         self.add_payment_source(source)
         self.add_payment_event('Settled', tradeAmt,
